import * as process from 'node:process';
import got from 'got';
import PQueue from 'p-queue';
import bole from 'bole';
import VERSION from '../version.js';
const logger = bole('collapsify:http');
class GotResponse {
    constructor(statusCode, contentType, buffer) {
        this.statusCode = statusCode;
        this.contentType = contentType;
        this.buffer = buffer;
    }
    getStatusCode() {
        return this.statusCode;
    }
    getContentType() {
        return this.contentType;
    }
    async getAsString() {
        return String(this.buffer);
    }
    async getAsArray() {
        return this.buffer;
    }
}
export default function makeClient(defaultHeaders) {
    const cache = new Map();
    const client = got.extend({
        headers: {
            'user-agent': `Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10; rv:60.0) Gecko/20100101 Firefox/60.0 Collapsify/${String(VERSION)} node/${String(process.version)}`,
            ...defaultHeaders,
        },
        responseType: 'buffer',
        timeout: { request: 2000 },
        retry: { limit: 5 },
    });
    const queue = new PQueue({
        concurrency: 8,
    });
    async function gotFetch(url) {
        const response = await queue.add(() => {
            logger.debug('Fetching %s.', url);
            return client.get(url, { cache });
        });
        if (response.isFromCache) {
            logger.debug('Retrieved %s from cache.', url);
        }
        return new GotResponse(response.statusCode, response.headers['content-type'], response.rawBody);
    }
    return gotFetch;
}
